/**
 * Conditional exports ordered from most specific to least specific.
 *
 * Conditions are matched based on order of the keys.
 */
export declare type ConditionalExports = {
    [condition: string]: ExportsEntry;
};
/**
 * Recursive exports entry.
 */
export declare type ExportsEntry = string | ConditionalExports | ExportsEntry[];
/**
 * Custom subpaths (with or without patterns).
 */
export declare type SubpathExports = Record<string, ExportsEntry | null>;
/**
 * The `"exports"` field of `package.json`.
 */
export declare type Exports = ExportsEntry | SubpathExports;
/**
 * Normalization options.
 */
export interface NormalizeExportsOptions {
    /**
     * Additional conditions that should be supported.
     *
     * By default, the following conditions are supported:
     * ```
     * ['node', 'import', 'default']
     * ```
     *
     * The order specified here does not matter. Conditions are always matched based on `"exports"` map's key order.
     */
    conditions?: string[];
    /**
     * Current working directory to resolve subpaths containing wildcard patterns.
     */
    cwd?: string;
}
/**
 * Normalizes the `"exports"` field of package.json and returns a flattened object.
 *
 * For example, for the below input:
 * ```json
 * {
 *   ".": {
 *     "import": "./dist/esm/index.js",
 *     "default": "./dist/index.js"
 *   },
 *   "./feature": "./dist/feature/index.js"
 * }
 * ```
 * following object is returned:
 * ```json
 * {
 *   ".": "./dist/esm/index.js",
 *   "./feature": "./dist/feature/index.js"
 * }
 * ```
 *
 * @param {Exports} exports Contents of `"exports"` field.
 * @param {NormalizeExportsOptions} options Options for normalization.
 * @returns {Record<string, string>} Normalized exports.
 */
export declare const normalizeExports: (exports: Exports, options?: NormalizeExportsOptions) => Record<string, string>;
