import { existsSync, readdirSync, statSync } from 'node:fs';
import path from 'node:path';
import picomatch from 'picomatch';
/**
 * Default supported conditions.
 */
const defaultConditions = ['node', 'import', 'default'];
/**
 * Checks if the `"exports"` field is well-formed.
 */
const checkExportsConfiguration = (exports) => {
    if (typeof exports === 'object') {
        if (Object.keys(exports).length === 0) {
            throw new Error('No entries found in exports map');
        }
        if (!Array.isArray(exports)) {
            const exportsKeys = Object.keys(exports);
            const keysStartingWithPeriod = exportsKeys.filter((key) => key.startsWith('.'));
            if (keysStartingWithPeriod.length !== 0 && keysStartingWithPeriod.length !== exportsKeys.length) {
                throw new Error('Invalid exports configuration');
            }
        }
    }
};
/**
 * Recursively matches nested conditions and returns a subpath (if found).
 */
const matchConditions = (exports, conditions) => {
    if (Array.isArray(exports)) {
        if (exports.length === 0) {
            return null;
        }
        let lastError;
        for (const subEntry of exports) {
            try {
                const value = matchConditions(subEntry, conditions);
                if (typeof value !== 'undefined') {
                    return value;
                }
            }
            catch (error) {
                lastError = error;
            }
        }
        if (typeof lastError === 'object') {
            throw lastError;
        }
    }
    else if (exports && typeof exports === 'object') {
        for (const [key, subEntry] of Object.entries(exports)) {
            if (conditions.has(key)) {
                const value = matchConditions(subEntry, conditions);
                if (typeof value !== 'undefined') {
                    return value;
                }
            }
        }
    }
    else if (typeof exports === 'string' && !exports.startsWith('./')) {
        throw new Error(`Invalid target '${exports}'`);
    }
    else {
        return exports;
    }
};
/**
 * Returns all subpath entries after expanding given wildcard pattern.
 */
const expandWildcardEntry = (key, value, cwd) => {
    const expr = picomatch.makeRe(value, {
        capture: true,
        noglobstar: true,
    });
    const directoryName = path.dirname(value);
    const directoryFullPath = path.join(cwd, directoryName);
    const expandedWildcardEntries = {};
    if (existsSync(directoryFullPath)) {
        const fileNames = readdirSync(directoryFullPath).filter((filePath) => statSync(path.join(directoryFullPath, filePath)).isFile());
        for (const fileName of fileNames) {
            const relativeFilePath = path.join(directoryName, fileName);
            const match = expr.exec(relativeFilePath);
            if (match?.[1]) {
                const [matchingPath, matchGroup] = match;
                const normalizedKey = key.replace('*', matchGroup);
                const normalizedFilePath = `./${matchingPath}`;
                expandedWildcardEntries[normalizedKey] = normalizedFilePath;
            }
        }
    }
    else {
        throw new Error(`Directory '${directoryFullPath}' not found`);
    }
    return expandedWildcardEntries;
};
/**
 * Returns flattened entries after matching conditions recursively and resolving wildcard patterns.
 */
const getFlattenedEntries = (exports, conditions, cwd) => {
    let flattenedEntries = {};
    const singleSubpath = matchConditions(exports, conditions);
    if (typeof singleSubpath === 'string') {
        flattenedEntries['.'] = singleSubpath;
    }
    else {
        for (const [key, subEntry] of Object.entries(exports)) {
            if (!key.startsWith('.')) {
                continue;
            }
            const value = matchConditions(subEntry, conditions);
            if (typeof value !== 'string') {
                continue;
            }
            if (key.includes('*') && cwd) {
                flattenedEntries = {
                    ...flattenedEntries,
                    ...expandWildcardEntry(key, value, cwd),
                };
            }
            else {
                flattenedEntries[key] = value;
            }
        }
    }
    return flattenedEntries;
};
/**
 * Normalizes the `"exports"` field of package.json and returns a flattened object.
 *
 * For example, for the below input:
 * ```json
 * {
 *   ".": {
 *     "import": "./dist/esm/index.js",
 *     "default": "./dist/index.js"
 *   },
 *   "./feature": "./dist/feature/index.js"
 * }
 * ```
 * following object is returned:
 * ```json
 * {
 *   ".": "./dist/esm/index.js",
 *   "./feature": "./dist/feature/index.js"
 * }
 * ```
 *
 * @param {Exports} exports Contents of `"exports"` field.
 * @param {NormalizeExportsOptions} options Options for normalization.
 * @returns {Record<string, string>} Normalized exports.
 */
export const normalizeExports = (exports, options) => {
    if (!exports) {
        throw new Error('Exports is not defined');
    }
    checkExportsConfiguration(exports);
    const conditions = new Set([...defaultConditions, ...(options?.conditions || [])]);
    const cwd = options?.cwd;
    const normalizedExports = getFlattenedEntries(exports, conditions, cwd);
    return normalizedExports;
};
